/*** GRAFICOS.C ***/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <conio.h>
#include <graphics.h>
#include <dos.h>
#include <string.h>
#include <dir.h>

float dist_obs  = 10000,  /* Distancia do observador `a origem               */
      dist_plano = 2000,  /* Distancia do plano de projecao `a origem        */
      a, b, c, d,         /* Equacao do plano de projecao: ax+by+cz+d = 0    */
      u1, u2, u3,         /* Vetor 1 tangente ao plano de projecao           */
      v1, v2, v3,         /* Vetor 2 tangente ao plano de projecao           */
      n1, n2, n3,                 /* Vetor normal                            */
      x0, y0, z0,                 /* Posicao do observador                   */
      nu = 40, nv = 40,           /* Subidivisoes do dominio                 */
      inc = 0.02,                 /* Incremento do parametro                 */
      P1, P2, P3,                 /* Ponto no plano de projecao              */
      o1 = 3, o2 = 1, o3 = 0.5,   /* Direcao do observador                   */
      O1 = 3, O2 = 1, O3 = 0.5;   /* Copias de o1, o2, o3                    */

/* ------------------------------------------------------------------------- */

#define  ESC            27
#define  SetaPCima      72
#define  SetaPBaixo     80
#define  SetaPEsquerda  75
#define  SetaPDireita   77
#define  PgUp           73
#define  PgDn           81
#define  TeclaF1        59
#define  TeclaF2        60
#define  TeclaF3        61
#define  TeclaF4        62
#define  TeclaF5        63
#define  TeclaF6        64
#define  TeclaF7        65
#define  TeclaF8        66
#define  TeclaF9        67
#define  TeclaF10       68
#define  TeclaF11       133
#define  TeclaF12       134
#define  CtrlSetaPBaixo 145
#define  CtrlSetaPCima  141
#define  CtrlF1         94
#define  CtrlF2         95
#define  CtrlF3         96
#define  CtrlF4         97
#define  CtrlF5         98
#define  CtrlF6         99
#define  CtrlF7         100
#define  CtrlF8         101
#define  CtrlF9         102
#define  CtrlF10        103
#define  AltF1          104
#define  AltF2          105
#define  AltF3          106
#define  AltF4          107
#define  AltF5          108
#define  AltF6          109
#define  AltF7          110
#define  AltF8          111
#define  AltF9          112
#define  AltF10         113

#define SQR(x)  ((x)*(x))

#define DISTANCIA(a, b, c, x, y, z) sqrt(SQR(a-x) + SQR(b-y) + SQR(c-z))

#define PROD_VET(V1, V2, V3, a1, a2, a3, b1, b2, b3) V1 = a2*b3-a3*b2; \
				   V2 = a3*b1-a1*b3; V3 = a1*b2-a2*b1

#define NORMA(V1, V2, V3)  (sqrt((V1)*(V1) + (V2)*(V2) + (V3)*(V3)))

/* ------------------------------------------------------------------------- */

int IniciaModoGrafico(void) {

/* Inicializa o modo grafico. Retorna 1 no caso de sucesso ou 0 em caso
   contrario. */

  int gm, gd = DETECT;

  registerbgidriver(EGAVGA_driver);
  initgraph(&gd, &gm, "");
  return (graphresult() == grOk);
}

/* ------------------------------------------------------------------------ */

int VetorNormal(float *N0, float *N1, float *N2, float (*f1)(float, float),
		float (*f2)(float, float), float (*f3)(float, float),
		float u0, float v0) {

/* Calcula (*N0, *N1, *N2) o vetor normal no  ponto  (u0, v0)  `a
   superficie f(u, v) = (f1(u, v), f2(u, v), f3(u, v)). Retorna 0
   se o vetor normal encontrado for nulo ou 1 em  caso contrario. */

  const float h = 1E-6;
  float a1, a2, a3, b1, b2, b3, norma_N;
  const float epsilon = 1e-6;

  /* Calculo de 2 vetores tangentes `a superficie f */

  a1 = (f1(u0 + h, v0) - f1(u0 - h, v0))/(2.0*h);
  a2 = (f2(u0 + h, v0) - f2(u0 - h, v0))/(2.0*h);
  a3 = (f3(u0 + h, v0) - f3(u0 - h, v0))/(2.0*h);

  b1 = (f1(u0, v0 + h) - f1(u0, v0 - h))/(2.0*h);
  b2 = (f2(u0, v0 + h) - f2(u0, v0 - h))/(2.0*h);
  b3 = (f3(u0, v0 + h) - f3(u0, v0 - h))/(2.0*h);

  /* Vetor normal a f */
  PROD_VET(*N0, *N1, *N2, a1, a2, a3, b1, b2, b3);

  norma_N = NORMA(*N0, *N1, *N2);
  if (norma_N < epsilon) return 0;
  return 1;
}

/* ------------------------------------------------------------------------- */

void DeterminaPlanoDeProjecao(void) {

/* Determina o  plano de  projecao ax + by + cz + d = 0, um ponto do plano
   (P1, P2, P3) e 2 vetores tangentes ao plano (u1, u2, u3) e (v1, v2, v3)
   perpendiculares entre si. */

  float N_o, prod_aux, t, N_u, N_v, compr_aux = 10;

  N_o = NORMA(o1, o2, o3);
  a = o1; b = o2; c = o3;
  o1 = o1/N_o; o2 = o2/N_o; o3 = o3/N_o;
  x0 = o1*dist_obs; y0 = o2*dist_obs; z0 = o3*dist_obs;
  P1 = o1*dist_plano; P2 = o2*dist_plano; P3 = o3*dist_plano;
  d = -a*P1 - b*P2 - c*P3;

  /* Determinacao de 2 vetores ortogonais no plano de projecao */

  if (o1 != 0) {
    t = 1/(1 - compr_aux*c/(a*x0 + b*y0 + c*z0 + d));
    u1 = x0 + t*(P1 - x0) - P1;
    u2 = y0 + t*(P2 - y0) - P2;
    u3 = z0 + t*(P3 - z0 + compr_aux) - P3;
    N_u = NORMA(u1, u2, u3);
    u1 = u1/N_u; u2 = u2/N_u; u3 = u3/N_u;

    t = 1/(1 - compr_aux*b/(a*x0 + b*y0 + c*z0 + d));
    v1 = x0 + t*(P1 - x0) - P1;
    v2 = y0 + t*(P2 - y0 + compr_aux) - P2;
    v3 = z0 + t*(P3 - z0) - P3;
    N_v = NORMA(v1, v2, v3);
    v1 = v1/N_v; v2 = v2/N_v; v3 = v3/N_v;
  }

  else if (o2 != 0) {
    t = 1/(1 - compr_aux*c/(a*x0 + b*y0 + c*z0 + d));
    u1 = x0 + t*(P1 - x0) - P1;
    u2 = y0 + t*(P2 - y0) - P2;
    u3 = z0 + t*(P3 - z0 + compr_aux) - P3;
    N_u = NORMA(u1, u2, u3);
    u1 = u1/N_u; u2 = u2/N_u; u3 = u3/N_u;

    t = 1/(1 - compr_aux*a/(a*x0 + b*y0 + c*z0 + d));
    v1 = x0 + t*(P1 - x0 + compr_aux) - P1;
    v2 = y0 + t*(P2 - y0) - P2;
    v3 = z0 + t*(P3 - z0) - P3;
    N_v = NORMA(v1, v2, v3);
    v1 = v1/N_v; v2 = v2/N_v; v3 = v3/N_v;
  }

  else if (o3 != 0) {
    t = 1/(1 - compr_aux*a/(a*x0 + b*y0 + c*z0 + d));
    u1 = x0 + t*(P1 - x0 + compr_aux) - P1;
    u2 = y0 + t*(P2 - y0) - P2;
    u3 = z0 + t*(P3 - z0) - P3;
    N_u = NORMA(u1, u2, u3);
    u1 = u1/N_u; u2 = u2/N_u; u3 = u3/N_u;

    t = 1/(1 - compr_aux*b/(a*x0 + b*y0 + c*z0 + d));
    v1 = x0 + t*(P1 - x0) - P1;
    v2 = y0 + t*(P2 - y0 + compr_aux) - P2;
    v3 = z0 + t*(P3 - z0) - P3;
    N_v = NORMA(v1, v2, v3);
    v1 = v1/N_v; v2 = v2/N_v; v3 = v3/N_v;
  }

  prod_aux = u1*v1 + u2*v2 + u3*v3;
  v1 = v1 - prod_aux*u1;
  v2 = v2 - prod_aux*u2;
  v3 = v3 - prod_aux*u3;
}

/* ------------------------------------------------------------------------- */

void ProjetaPonto(float x1, float y1, float z1, float *x, float *y) {

/* Determina as coordenadas do ponto (x1, y1, z1) no plano de projecao. Deve
   ser feita antes uma chamada a DeterminaPlanoDeProjecao(). */

  float q1, q2, q3, t;

  t = 1/(1 - (a*x1 + b*y1 + c*z1 + d)/(a*x0 + b*y0 + c*z0 + d));
  q1 = x0 + t*(x1 - x0);
  q2 = y0 + t*(y1 - y0);
  q3 = z0 + t*(z1 - z0);
  *x = (q1 - P1)*v1 + (q2 - P2)*v2 + (q3 - P3)*v3;
  *y = (q1 - P1)*u1 + (q2 - P2)*u2 + (q3 - P3)*u3;
}

/* ------------------------------------------------------------------------- */

void DesenhaEixos(int tam, int cor) {

/* Desenha os eixos x, y, z usando tamanho e cor especificados */

  float X0, Y0, X, Y;
  int cor_aux, x_x, x_y, y_x, y_y, z_x, z_y;

  cor_aux = getcolor();
  setcolor(cor);
  ProjetaPonto(0, 0, 0, &X0, &Y0);
  ProjetaPonto(tam, 0, 0, &X, &Y);
  line((int) (A + X0*TAM), (int) (B - Y0*TAM),
       (int) (A + X*TAM), (int) (B - Y*TAM));
  x_x = (int) (A + X*TAM); x_y = (int) (B - Y*TAM);
  ProjetaPonto(0, tam, 0, &X, &Y);
  line((int) (A + X0*TAM), (int) (B - Y0*TAM),
       (int) (A + X*TAM), (int) (B - Y*TAM));
  y_x = (int) (A + X*TAM); y_y = (int) (B - Y*TAM);
  ProjetaPonto(0, 0, tam, &X, &Y);
  line((int) (A + X0*TAM), (int) (B - Y0*TAM),
       (int) (A + X*TAM), (int) (B - Y*TAM));
  z_x = (int) (A + X*TAM); z_y = (int) (B - Y*TAM);

  setcolor(cor_aux);
  outtextxy(x_x, x_y, "x");
  outtextxy(y_x, y_y, "y");
  outtextxy(z_x, z_y, "z");
}

/* ------------------------------------------------------------------------- */

void DesenhaPonto(float x, float y, float z) {

/* Desenha a projecao do ponto (x, y, z) no plano de projecao. Deve
   ser feita antes uma chamada a DeterminaPlanoDeProjecao(). */

  float X, Y, q1, q2, q3, t;

  t = 1/(1 - (a*x + b*y + c*z + d)/(a*x0 + b*y0 + c*z0 + d));
  q1 = x0 + t*(x - x0);
  q2 = y0 + t*(y - y0);
  q3 = z0 + t*(z - z0);
  X = (q1 - P1)*v1 + (q2 - P2)*v2 + (q3 - P3)*v3;
  Y = (q1 - P1)*u1 + (q2 - P2)*u2 + (q3 - P3)*u3;
  putpixel((int) (A + X*TAM), (int) (B - Y*TAM), getcolor());
}

/* ------------------------------------------------------------------------- */

void DesenhaPontoCor(float x, float y, float z, int cor) {

/* Desenha a projecao do ponto (x, y, z) no plano de projecao. Deve
   ser feita antes uma chamada a DeterminaPlanoDeProjecao(). E' usa-
   da a cor fornecida. */

  float X, Y, q1, q2, q3, t;

  t = 1/(1 - (a*x + b*y + c*z + d)/(a*x0 + b*y0 + c*z0 + d));
  q1 = x0 + t*(x - x0);
  q2 = y0 + t*(y - y0);
  q3 = z0 + t*(z - z0);
  X = (q1 - P1)*v1 + (q2 - P2)*v2 + (q3 - P3)*v3;
  Y = (q1 - P1)*u1 + (q2 - P2)*u2 + (q3 - P3)*u3;
  putpixel((int) (A + X*TAM), (int) (B - Y*TAM), cor);
}

/* ------------------------------------------------------------------------- */

void DesenhaSuperficie(float (*f) (float, float), float (*g) (float, float),
			  float (*h) (float, float), float umin, float umax,
						     float vmin, float vmax) {

/* Desenha a superficie (f, g, h) com pontos isolados */

  float u, v;

  for (v = vmin; v < vmax; v += (vmax - vmin)/nv) {
    for (u = umin; u < umax; u += inc)
      DesenhaPonto(f(u, v), g(u, v), h(u, v));
    u = umax;
    DesenhaPonto(f(u, v), g(u, v), h(u, v));
  }
  v = vmax;
  for (u = umin; u < umax; u += inc)
    DesenhaPonto(f(u, v), g(u, v), h(u, v));
  u = umax;
  DesenhaPonto(f(u, v), g(u, v), h(u, v));

  for (u = umin; u < umax; u += (umax - umin)/nu) {
    for (v = vmin; v < vmax; v += inc)
      DesenhaPonto(f(u, v), g(u, v), h(u, v));
    v = vmax;
    DesenhaPonto(f(u, v), g(u, v), h(u, v));
  }
  u = umax;
  for (v = vmin; v < vmax; v += inc)
    DesenhaPonto(f(u, v), g(u, v), h(u, v));
  v = vmax;
  DesenhaPonto(f(u, v), g(u, v), h(u, v));
}

/* ------------------------------------------------------------------------- */

void DesenhaSuperficie2(float (*f) (float, float), float (*g) (float, float),
			  float (*h) (float, float), float umin, float umax,
						     float vmin, float vmax) {

/* Desenha a superficie (f, g, h) usando pequenos segmentos de reta para
   ligar os pontos */

  float X, Y, u, v, x, y, z, P1, P2, d;

  d = (vmax - vmin)/nv;
  for (v = vmin; v < vmax; v += d) {
    u = umin;
    x = f(u, v); y = g(u, v); z = h(u, v);
    ProjetaPonto(x, y, z, &X, &Y);
    P1 = A + X*TAM; P2 = B - Y*TAM;
    for (u = umin; u < umax; u += inc) {
      x = f(u, v); y = g(u, v); z = h(u, v);
      ProjetaPonto(x, y, z, &X, &Y);
      line((int) (A + X*TAM), (int) (B - Y*TAM), (int) P1, (int) P2);
      P1 = A + X*TAM; P2 = B - Y*TAM;
    }
    u = umax;
    x = f(u, v); y = g(u, v); z = h(u, v);
    ProjetaPonto(x, y, z, &X, &Y);
    line((int) (A + X*TAM), (int) (B - Y*TAM), (int) P1, (int) P2);
  }
  u = umin; v = vmax;
  x = f(u, v); y = g(u, v); z = h(u, v);
  ProjetaPonto(x, y, z, &X, &Y);
  P1 = A + X*TAM; P2 = B - Y*TAM;
  for (u = umin; u < umax; u += inc) {
    x = f(u, v); y = g(u, v); z = h(u, v);
    ProjetaPonto(x, y, z, &X, &Y);
    line((int) (A + X*TAM), (int) (B - Y*TAM), (int) P1, (int) P2);
    P1 = A + X*TAM; P2 = B - Y*TAM;
  }
  u = umax;
  x = f(u, v); y = g(u, v); z = h(u, v);
  ProjetaPonto(x, y, z, &X, &Y);
  line((int) (A + X*TAM), (int) (B - Y*TAM), (int) P1, (int) P2);

  d = (umax - umin)/nu;
  for (u = umin; u < umax; u += d) {
    v = vmin;
    x = f(u, v); y = g(u, v); z = h(u, v);
    ProjetaPonto(x, y, z, &X, &Y);
    P1 = A + X*TAM; P2 = B - Y*TAM;
    for (v = vmin; v < vmax; v += inc) {
      x = f(u, v); y = g(u, v); z = h(u, v);
      ProjetaPonto(x, y, z, &X, &Y);
      line((int) (A + X*TAM), (int) (B - Y*TAM), (int) P1, (int) P2);
      P1 = A + X*TAM; P2 = B - Y*TAM;
    }
    v = vmax;
    x = f(u, v); y = g(u, v); z = h(u, v);
    ProjetaPonto(x, y, z, &X, &Y);
    line((int) (A + X*TAM), (int) (B - Y*TAM), (int) P1, (int) P2);
    P1 = A + X*TAM; P2 = B - Y*TAM;
  }
  v = vmin; u = umax;
  x = f(u, v); y = g(u, v); z = h(u, v);
  ProjetaPonto(x, y, z, &X, &Y);
  P1 = A + X*TAM; P2 = B - Y*TAM;
  for (v = vmin; v < vmax; v += inc) {
    x = f(u, v); y = g(u, v); z = h(u, v);
    ProjetaPonto(x, y, z, &X, &Y);
    line((int) (A + X*TAM), (int) (B - Y*TAM), (int) P1, (int) P2);
    P1 = A + X*TAM; P2 = B - Y*TAM;
  }
  v = vmax;
  x = f(u, v); y = g(u, v); z = h(u, v);
  ProjetaPonto(x, y, z, &X, &Y);
  line((int) (A + X*TAM), (int) (B - Y*TAM), (int) P1, (int) P2);
}

/* ------------------------------------------------------------------------- */
/*
void DesenhaSuperficie3(float (*f) (float, float), float (*g) (float, float),
     float (*h) (float, float), float umin, float umax,
     float vmin, float vmax, int cor_frente, int cor_tras, int op) {

/* Desenha a superficie (f, g, h) com pontos isolados. Sao usadas cores
   para pintar os pontos pela frente e por tras da superficie.
   Se op > 0, sao desenhadas so' as curvas coordenadas u,
   se op < 0, sao desenhadas so' as curvas coordenadas v,
   se op == 0, sao desenhadas  as curvas coordenadas u e v */

  float u, v, n1, n2, n3;

  if (op >= 0) {
    for (v = vmin; v < vmax; v += (vmax - vmin)/nv) {
      for (u = umin; u < umax; u += inc) {
	VetorNormal(&n1, &n2, &n3, f, g, h, u, v);
	if (o1*n1 + o2*n2 + o3*n3 >= 0)
	  DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor_frente);
	else
	  DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor_tras);
      }
      u = umax;
      VetorNormal(&n1, &n2, &n3, f, g, h, u, v);
      if (o1*n1 + o2*n2 + o3*n3 >= 0)
	DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor_frente);
      else
	DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor_tras);
    }
    v = vmax;
    for (u = umin; u < umax; u += inc) {
      VetorNormal(&n1, &n2, &n3, f, g, h, u, v);
      if (o1*n1 + o2*n2 + o3*n3 >= 0)
	DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor_frente);
      else
	DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor_tras);
    }
    u = umax;
    VetorNormal(&n1, &n2, &n3, f, g, h, u, v);
    if (o1*n1 + o2*n2 + o3*n3 >= 0)
      DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor_frente);
    else
      DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor_tras);
  }

  if (op <= 0) {
    for (u = umin; u < umax; u += (umax - umin)/nu) {
      for (v = vmin; v < vmax; v += inc) {
	VetorNormal(&n1, &n2, &n3, f, g, h, u, v);
	if (o1*n1 + o2*n2 + o3*n3 >= 0)
	  DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor_frente);
	else
	  DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor_tras);
      }
      v = vmax;
      VetorNormal(&n1, &n2, &n3, f, g, h, u, v);
      if (o1*n1 + o2*n2 + o3*n3 >= 0)
	DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor_frente);
      else
	DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor_tras);
    }
    u = umax;
    for (v = vmin; v < vmax; v += inc) {
      VetorNormal(&n1, &n2, &n3, f, g, h, u, v);
      if (o1*n1 + o2*n2 + o3*n3 >= 0)
	DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor_frente);
      else
	DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor_tras);
    }
    v = vmax;
    VetorNormal(&n1, &n2, &n3, f, g, h, u, v);
    if (o1*n1 + o2*n2 + o3*n3 >= 0)
      DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor_frente);
    else
      DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor_tras);
  }

}

*/

/* ------------------------------------------------------------------------- */

int InterceptaReta( long a1, long b1, long a2, long b2,
		    long a3, long b3, long a4, long b4) {

/* Verifica se o segmento de reta (a1, b1) - (a2, b2) intercepta o segmento
   (a3, b3) - (a4, b4)  em  um  unico  ponto.   Nao   sao  consideradas  as
   extremidades dos segmentos. */

  float det, t, s;

  det = (a2 - a1)*(b3 - b4) - (a3 - a4)*(b2 - b1);
  if (det == 0) return 0;

  t = ((a3 - a1)*(b3 - b4) - (a3 - a4)*(b3 - b1))/det;
  s = ((a2 - a1)*(b3 - b1) - (a3 - a1)*(b2 - b1))/det;

  return (0 < t && t < 1 && 0 < s && s < 1);

}

/* ------------------------------------------------------------------------- */

int FechoConvexo(int *a1, int *b1, int *a2, int *b2,
		  int *a3, int *b3, int *a4, int *b4) {

/* Determina o fecho convexo do poligono
		(a1, b1) - (a2, b2) - (a3, b3) - (a4, b4)          */

  int a_aux, b_aux;

  if (InterceptaReta(*a1, *b1, *a2, *b2, *a3, *b3, *a4, *b4)) {
    a_aux = *a1; b_aux = *b1;
    *a1 = *a4;   *b1 = *b4;
    *a4 = a_aux; *b4 = b_aux;
    return 14;
  }

  if (InterceptaReta(*a1, *b1, *a4, *b4, *a3, *b3, *a2, *b2)) {
    a_aux = *a1; b_aux = *b1;
    *a1 = *a2;   *b1 = *b2;
    *a2 = a_aux; *b2 = b_aux;
    return 12;
  }

  return 0;
}

/* ------------------------------------------------------------------------- */

/*
void DesenhaSuperficie4(float (*f1) (float, float), float (*f2) (float, float),
			    float (*f3) (float, float), float umin, float umax,
			    float vmin, float vmax) {

  /* Desenha a superficie (f1, f2, f3) usando pequenos quadrilateros */

  float u, v, xp1, yp1, zp1, xp2, yp2, zp2, xp3, yp3, zp3, xp4, yp4, zp4,
	X, Y;
  int xq1, xq2, xq3, xq4, yq1, yq2, yq3, yq4;

  for (u = umin + (umax - umin)/nu; u <= umax + (umax - umin)/(2*nu);
						       u += (umax - umin)/nu)
    for (v = vmin + (vmax - vmin)/nv; v <= vmax + (vmax - vmin)/(2*nv);
						       v += (vmax - vmin)/nv) {

      xp1 = f1(u, v);
      yp1 = f2(u, v);
      zp1 = f3(u, v);

      xp2 = f1(u - (umax - umin)/nu, v);
      yp2 = f2(u - (umax - umin)/nu, v);
      zp2 = f3(u - (umax - umin)/nu, v);

      xp3 = f1(u, v - (vmax - vmin)/nv);
      yp3 = f2(u, v - (vmax - vmin)/nv);
      zp3 = f3(u, v - (vmax - vmin)/nv);

      xp4 = f1(u - (umax - umin)/nu, v - (vmax - vmin)/nv);
      yp4 = f2(u - (umax - umin)/nu, v - (vmax - vmin)/nv);
      zp4 = f3(u - (umax - umin)/nu, v - (vmax - vmin)/nv);

      ProjetaPonto(xp1, yp1, zp1, &X, &Y);
      xq1 = (int) (A + X*TAM); yq1 = (int) (B - Y*TAM);
      ProjetaPonto(xp2, yp2, zp2, &X, &Y);
      xq2 = (int) (A + X*TAM); yq2 = (int) (B - Y*TAM);
      ProjetaPonto(xp3, yp3, zp3, &X, &Y);
      xq3 = (int) (A + X*TAM); yq3 = (int) (B - Y*TAM);
      ProjetaPonto(xp4, yp4, zp4, &X, &Y);
      xq4 = (int) (A + X*TAM); yq4 = (int) (B - Y*TAM);

      FechoConvexo(&xq1, &yq1, &xq2, &yq2, &xq3, &yq3, &xq4, &yq4);

      line (xq1, yq1, xq2, yq2);
      line (xq3, yq3, xq2, yq2);
      line (xq3, yq3, xq4, yq4);
      line (xq4, yq4, xq1, yq1);
    }
}

*/
/* ------------------------------------------------------------------------- */

int SalvaImagem(char *NomeArquivo) {

/* Salva em disco a tela inteira em modo grafico */

  int coluna, linha, maxx, maxy;
  char cor;
  FILE *arq;
  int k = 0;
  char arq_imagem[80];

  strcpy(arq_imagem, NomeArquivo);
  while(arq_imagem[k] != '\0' && arq_imagem[k] != '.') k++;
  arq_imagem[k] = '\0';
  strcat(arq_imagem, ".IMG");

  maxx = getmaxx();
  maxy = getmaxy();
  if ((arq = fopen(arq_imagem, "wb")) == NULL) return 0;
  for (linha = 0; linha <= maxy; linha++)
    for (coluna = 0; coluna <= maxx; coluna++) {
      cor = (char) getpixel(coluna, linha);
      putc(cor, arq);
    }
  fclose(arq);
  return 1;
}

/* ------------------------------------------------------------------------- */

int LeImagem(char *NomeArquivo) {

/* Le tela salva em disco com SalvaImagem() */

  int maxx, coluna, linha;
  char cor;
  long i = 0;
  FILE *arq;
  int k = 0;
  char arq_imagem[80];

  strcpy(arq_imagem, NomeArquivo);
  while(arq_imagem[k] != '\0' && arq_imagem[k] != '.') k++;
  arq_imagem[k] = '\0';
  strcat(arq_imagem, ".IMG");

  maxx = getmaxx();
  if ((arq = fopen(arq_imagem, "rb")) == NULL) return 0;
  maxx = getmaxx();

  do {
    if ((cor = getc(arq)) != EOF) {
      coluna = (int) (i % (maxx + 1));
      linha =  (int) (i/(maxx + 1));
      putpixel(coluna, linha, cor);
    }
    i++;
  } while (cor != EOF);

  fclose(arq);
  return 1;
}

/* ------------------------------------------------------------------------- */

int SalvaConfiguracaoDoGrafico(char  *NomeArquivo,  int  cor_f1,  int  cor_f2,
  int cor1, int cor2, int cor3, int cor4, int padrao1, int padrao2, int opcao,
  float xmin,  float xmax,  float ymin,  float ymax,  float zmax,  float zmin,
  float smax, float s1,  float s2,  float s3,  float s4,  float s5,  float s6,
  float s7, float s8) {

  FILE *arq;
  char arq_config[80];
  int k = 0;

  strcpy(arq_config, NomeArquivo);
  while(arq_config[k] != '\0' && arq_config[k] != '.') k++;
  NomeArquivo[k] = '\0';
  strcat(arq_config, ".CFG");

  if ((arq = fopen(arq_config, "wb")) == NULL) return 0;
  fwrite(&cor_f1, sizeof(int), 1, arq);
  fwrite(&cor_f2, sizeof(int), 1, arq);
  fwrite(&cor1, sizeof(int), 1, arq);
  fwrite(&cor2, sizeof(int), 1, arq);
  fwrite(&cor3, sizeof(int), 1, arq);
  fwrite(&cor4, sizeof(int), 1, arq);
  fwrite(&padrao1, sizeof(int), 1, arq);
  fwrite(&padrao2, sizeof(int), 1, arq);
  fwrite(&opcao, sizeof(int), 1, arq);

  fwrite(&xmin, sizeof(float), 1, arq);
  fwrite(&xmax, sizeof(float), 1, arq);
  fwrite(&ymin, sizeof(float), 1, arq);
  fwrite(&ymax, sizeof(float), 1, arq);
  fwrite(&zmin, sizeof(float), 1, arq);
  fwrite(&zmax, sizeof(float), 1, arq);
  fwrite(&smax, sizeof(float), 1, arq);
  fwrite(&s1, sizeof(float), 1, arq);
  fwrite(&s2, sizeof(float), 1, arq);
  fwrite(&s3, sizeof(float), 1, arq);
  fwrite(&s4, sizeof(float), 1, arq);
  fwrite(&s5, sizeof(float), 1, arq);
  fwrite(&s6, sizeof(float), 1, arq);
  fwrite(&s7, sizeof(float), 1, arq);
  fwrite(&s8, sizeof(float), 1, arq);

  fwrite(&dist_obs, sizeof(float), 1, arq);
  fwrite(&dist_plano, sizeof(float), 1, arq);
  fwrite(&a, sizeof(float), 1, arq);
  fwrite(&b, sizeof(float), 1, arq);
  fwrite(&c, sizeof(float), 1, arq);
  fwrite(&d, sizeof(float), 1, arq);
  fwrite(&u1, sizeof(float), 1, arq);
  fwrite(&u2, sizeof(float), 1, arq);
  fwrite(&u3, sizeof(float), 1, arq);
  fwrite(&v1, sizeof(float), 1, arq);
  fwrite(&v2, sizeof(float), 1, arq);
  fwrite(&v3, sizeof(float), 1, arq);
  fwrite(&o1, sizeof(float), 1, arq);
  fwrite(&o2, sizeof(float), 1, arq);
  fwrite(&o3, sizeof(float), 1, arq);
  fwrite(&A, sizeof(float), 1, arq);
  fwrite(&B, sizeof(float), 1, arq);
  fwrite(&TAM, sizeof(float), 1, arq);
  fwrite(&inc, sizeof(float), 1, arq);
  fwrite(&x0, sizeof(float), 1, arq);
  fwrite(&y0, sizeof(float), 1, arq);
  fwrite(&z0, sizeof(float), 1, arq);
  fwrite(&P1, sizeof(float), 1, arq);
  fwrite(&P2, sizeof(float), 1, arq);
  fwrite(&P3, sizeof(float), 1, arq);
  fwrite(&nu, sizeof(float), 1, arq);
  fwrite(&nv, sizeof(float), 1, arq);

  fclose(arq);
  return 1;
}

/* ------------------------------------------------------------------------- */

int LeConfiguracaoDoGrafico(char  *NomeArquivo,   int  *cor_f1,  int  *cor_f2,
  int *cor1,  int *cor2,  int *cor3,  int *cor4,  int *padrao1,  int *padrao2,
  int *opcao, float *xmin, float *xmax, float *ymin, float *ymax, float *zmax,
  float *zmin,  float  *smax, float *s1,  float  *s2,  float  *s3,  float *s4,
  float *s5, float *s6, float *s7, float *s8) {

  FILE *arq;
  char arq_config[80];
  int k = 0;

  strcpy(arq_config, NomeArquivo);
  while(arq_config[k] != '\0' && arq_config[k] != '.') k++;
  arq_config[k] = '\0';
  strcat(arq_config, ".CFG");

  if ((arq = fopen(arq_config, "rb")) == NULL) return 0;
  fread(cor_f1, sizeof(int), 1, arq);
  fread(cor_f2, sizeof(int), 1, arq);
  fread(cor1, sizeof(int), 1, arq);
  fread(cor2, sizeof(int), 1, arq);
  fread(cor3, sizeof(int), 1, arq);
  fread(cor4, sizeof(int), 1, arq);
  fread(padrao1, sizeof(int), 1, arq);
  fread(padrao2, sizeof(int), 1, arq);
  fread(opcao, sizeof(int), 1, arq);

  fread(xmin, sizeof(float), 1, arq);
  fread(xmax, sizeof(float), 1, arq);
  fread(ymin, sizeof(float), 1, arq);
  fread(ymax, sizeof(float), 1, arq);
  fread(zmin, sizeof(float), 1, arq);
  fread(zmax, sizeof(float), 1, arq);
  fread(smax, sizeof(float), 1, arq);
  fread(s1, sizeof(float), 1, arq);
  fread(s2, sizeof(float), 1, arq);
  fread(s3, sizeof(float), 1, arq);
  fread(s4, sizeof(float), 1, arq);
  fread(s5, sizeof(float), 1, arq);
  fread(s6, sizeof(float), 1, arq);
  fread(s7, sizeof(float), 1, arq);
  fread(s8, sizeof(float), 1, arq);

  fread(&dist_obs, sizeof(float), 1, arq);
  fread(&dist_plano, sizeof(float), 1, arq);
  fread(&a, sizeof(float), 1, arq);
  fread(&b, sizeof(float), 1, arq);
  fread(&c, sizeof(float), 1, arq);
  fread(&d, sizeof(float), 1, arq);
  fread(&u1, sizeof(float), 1, arq);
  fread(&u2, sizeof(float), 1, arq);
  fread(&u3, sizeof(float), 1, arq);
  fread(&v1, sizeof(float), 1, arq);
  fread(&v2, sizeof(float), 1, arq);
  fread(&v3, sizeof(float), 1, arq);
  fread(&o1, sizeof(float), 1, arq);
  fread(&o2, sizeof(float), 1, arq);
  fread(&o3, sizeof(float), 1, arq);
  fread(&A, sizeof(float), 1, arq);
  fread(&B, sizeof(float), 1, arq);
  fread(&TAM, sizeof(float), 1, arq);
  fread(&inc, sizeof(float), 1, arq);
  fread(&x0, sizeof(float), 1, arq);
  fread(&y0, sizeof(float), 1, arq);
  fread(&z0, sizeof(float), 1, arq);
  fread(&P1, sizeof(float), 1, arq);
  fread(&P2, sizeof(float), 1, arq);
  fread(&P3, sizeof(float), 1, arq);
  fread(&nu, sizeof(float), 1, arq);
  fread(&nv, sizeof(float), 1, arq);

  fclose(arq);
  return 1;
}

/* ------------------------------------------------------------------------- */

void DesenhaSuperficie5(float (*f) (float, float), float (*g) (float, float),
     float (*h) (float, float), float umin, float umax,
     float vmin, float vmax, int cor1, int cor2) {

  float X, Y, u, v;
  int horizonte_max[2000], horizonte_min[2000], maxx, i, x, y;

  maxx = getmaxx();
  for (i = 0; i <= maxx; i++) {
    horizonte_max[i] = 9999;
    horizonte_min[i] = -9999;
  }

  for (u = umax; u > umin; u -= (umax - umin)/nu) {
    for (v = vmin; v < vmax; v += inc) {
      ProjetaPonto(f(u, v), g(u, v), h(u, v), &X, &Y);
      x = (int) (A + X*TAM); y = (int) (B - Y*TAM);
      if (x >= 0 && x <= maxx) {
	if (y < horizonte_max[x]) {
	  horizonte_max[x] = y;
	  DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor1);
	}
	if (y > horizonte_min[x]) {
	  horizonte_min[x] = y;
	  DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor2);
	}
      }
    }

    v = vmax;
    ProjetaPonto(f(u, v), g(u, v), h(u, v), &X, &Y);
    x = (int) (A + X*TAM); y = (int) (B - Y*TAM);
    if (x >= 0 && x <= maxx) {
      if (y < horizonte_max[x]) {
	horizonte_max[x] = y;
	DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor1);
      }
      if (y > horizonte_min[x]) {
	horizonte_min[x] = y;
	DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor2);
      }
    }
  }

  u = umin;
  for (v = vmin; v < vmax; v += inc) {
    ProjetaPonto(f(u, v), g(u, v), h(u, v), &X, &Y);
    x = (int) (A + X*TAM); y = (int) (B - Y*TAM);
    if (x >= 0 && x <= maxx) {
      if (y < horizonte_max[x]) {
	horizonte_max[x] = y;
	DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor1);
      }
      if (y > horizonte_min[x]) {
	horizonte_min[x] = y;
	DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor2);
      }
    }
  }

  v = vmax;
  ProjetaPonto(f(u, v), g(u, v), h(u, v), &X, &Y);
  x = (int) (A + X*TAM); y = (int) (B - Y*TAM);
  if (x >= 0 && x <= maxx) {
    if (y < horizonte_max[x]) {
      horizonte_max[x] = y;
      DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor1);
    }
    if (y > horizonte_min[x]) {
      horizonte_min[x] = y;
      DesenhaPontoCor(f(u, v), g(u, v), h(u, v), cor2);
    }
  }
}

/* ------------------------------------------------------------------------- */

/*** FIM DE GRAFICOS.C ***/
